<?php
/**
 * @package     Joomla.Administrator
 * @subpackage  com_httpcsvalidation
 *
 * @copyright   Copyright (C) 2005 - 2015 Open Source Matters, Inc. All rights reserved.
 * @license     GNU General Public License version 2 or later; see https://www.gnu.org/licenses/quick-guide-gplv3.en.html
 */
 
// No direct access to this file
defined('_JEXEC') or die('Restricted access');
 
/**
 * General Controller of Httpcsvalidation component
 *
 * @package     Joomla.Administrator
 * @subpackage  com_httpcsvalidation
 * @since       0.0.7
 */

class HttpcsvalidationController extends JControllerLegacy
{
    protected $url            = "";
    protected $function       = "";
    protected $company        = "";
    protected $nameU           = "";
    protected $phone          = "";
    protected $email          = "";
    protected $messageSuccess = "";
    protected $messageError   = "";
    protected $secondToken    = "";
    protected $render         = "init";
    protected $fail           = 0;
    protected $event          = "";
    protected $pathModule     = "";
    protected $objFormHandler = "";
    protected $fileName       = "";
    protected $fileContent    = "";
    protected $dirname        = "";
    protected $securedToken   = "";
    
    //Paths to HTTPCS
    private $PATHHTTPCS             = '';
    private $PATHCHECKURLEMAIL      = '';
    private $PATHCHECKEMAILPASSWORD = '';
    private $PATHCHECKFILE          = '';
    private $PATHCHECKFILEAGAIN     = '';
    private $PATHCONNECTION         = '/user/login';
    private $PATHFORGOTPASSWORD     = '/user/reset-password';
    
    /**
    * Method to display a view.
    *
    * @param   boolean  $cachable   If true, the view output will be cached
    * @param   mixed    $urlparams  An array of safe url parameters and their variable types, for valid values see {@link JFilterInput::clean()}.
    *
    * @return   JController This object to support chaining.
    *
    * @since    1.5
    */
    public function display($cachable = false, $urlparams = false)
    {
        include 'classes/httpcsFormHandler.php';
        include 'httpcsConfig.php';
        $this->objFormHandler    = new HttpcsFormHandler();
        $this->url = JURI::root();
        $this->pathModule = JUri::base() . 'components/com_httpcsvalidation/';
        
        $session = JFactory::getSession();
        $securedToken = $session->get('securedToken');
        if( $securedToken == null || ($securedToken != null && empty($securedToken)) ){
            $version = explode('.',PHP_VERSION);
            $PHP_VERSION_ID = ($version[0] * 10000 + $version[1] * 100 + $version[2]);
            if($PHP_VERSION_ID < 700000){
                $session->set('securedToken', hash('sha512', bin2hex(mt_getrandmax())));
            }else{
                $session->set('securedToken', hash('sha512', bin2hex(random_bytes(100))));
            }
        }
        $this->securedToken = $session->get('securedToken');
        
        $this->init();
        
        $view = JFactory::getApplication()->input->getCmd('view', $this->render);
        JFactory::getApplication()->input->set('view', $view);    

        $oView = $this->getView($this->render,'html');
		
        $oView->securedToken = $this->securedToken;
        $oView->nameU = htmlentities($this->nameU);
        $oView->email = htmlentities($this->email);
        $oView->company = htmlentities($this->company);
        $oView->function = htmlentities($this->function);
        $oView->phone = htmlentities($this->phone);       
        $oView->url = $this->url;       
        $oView->event = $this->event;       
        $oView->secondToken = $this->secondToken;       
        $oView->pathModule = $this->pathModule;       
        $oView->fail = $this->fail;       
        $oView->notices = $this->getNotices();   
        
        if($this->render == 'validated' || $this->render == 'success' || $this->render == 'successco'){
            $lang = JFactory::getLanguage();
            $languages = JLanguageHelper::getLanguages('lang_code');
            $languageTag = $lang->getTag();
            $languageCode = $languages[ $languageTag ]->sef;
            if($languageCode == 'fr'){
                    $locale = 'fr';
                    $this->PATHCONNECTION = '/utilisateur/connexion';
            }else{
                    $locale = 'en';
            }
            $oView->locale = $locale;
            $oView->PATHHTTPCS = $this->PATHHTTPCS;
            $oView->PATHCONNECTION = $this->PATHCONNECTION;
        }
        
        if($this->render == 'writingerror'){
            $oView->fileName = $this->fileName;
            $oView->fileContent = $this->fileContent;
            $oView->dirname = $this->dirname;
        }
        
        if($this->render == 'init'){
            $lang = JFactory::getLanguage();
            $languages = JLanguageHelper::getLanguages('lang_code');
            $languageTag = $lang->getTag();
            $languageCode = $languages[ $languageTag ]->sef;
            if($languageCode == 'fr'){
                    $locale = 'fr';
                    $this->PATHFORGOTPASSWORD = '/utilisateur/reinitialiser-password';
            }else{
                    $locale = 'en';
            }
            $oView->locale = $locale;
            $oView->PATHHTTPCS = $this->PATHHTTPCS;
            $oView->PATHFORGOTPASSWORD = $this->PATHFORGOTPASSWORD;
        }

        parent::display($cachable, $urlparams);

        return $this;
    }
    
    public function init(){
        $isValid = JComponentHelper::getParams('com_httpcsvalidation')->get('validated');
        if (!$this->isCurl()) {
            $this->render = 'curldisabled';
        } else {
            $this->objFormHandler->handleForm(filter_input_array(INPUT_POST));            
            $httpcsToken = filter_input(INPUT_POST, 'securedToken');
            if (is_object($this->objFormHandler) && $_SERVER['REQUEST_METHOD'] === "POST" && $isValid !== '1' && $this->securedToken == $httpcsToken) {
                //Creation
                $httpcsCrea = filter_input(INPUT_POST, 'httpcs_crea');
                if ($httpcsCrea === 'Y') {
                    $this->creation($this->objFormHandler);
                }
                //Connexion
                $httpcsCo = filter_input(INPUT_POST, 'httpcs_co');
                if ($httpcsCo === 'Y') {
                    $this->connexion($this->objFormHandler);
                }
                //Retry
                $httpcsRetry = filter_input(INPUT_POST, 'httpcs_retry');
                if ($httpcsRetry === 'Y') {
                    $this->retry($this->objFormHandler);
                }
            }
        }

        if ($isValid === '1') {
            $this->render = "validated";
        }
    }
    
    /*
     * Handles creation
     */
    public function creation($objFormHandler) {
        $aFormHandler   = $objFormHandler->objToArray();
        $response[]     = $this->curlRequest($this->PATHCHECKURLEMAIL, $aFormHandler);
        $this->email    = $objFormHandler->getEmail();
        $this->nameU    = urldecode($objFormHandler->getName());
        $this->function = urldecode($objFormHandler->getFunction());
        $this->company  = urldecode($objFormHandler->getCompany());
        $this->phone    = urldecode($objFormHandler->getPhone());
        if($response[0] != 'error'){
            if($this->handleResponse($response[0])){
                if (json_decode($response[0])->etat) {
                    //return   C:\wamp64\www\joomla\administrator/components/com_httpcsvalidation/public/
                    $uploads      = JPATH_COMPONENT.'/public/';
                    //  return /administrator/components/com_httpcsvalidation/public/
                    $relativePath = str_replace(JPATH_SITE, '', $uploads);
                    $response[]   = $this->curlRequest($this->PATHCHECKFILE, array('url' => $aFormHandler['url'], 'relativePath' => $relativePath, 'token' => json_decode($response[0])->token, 'email' => $this->email, 'event' => 'creation'));
                    if($response[1] != 'error'){
                        if (!json_decode($response[1])->etat) {
                            $this->render      = "retry";
                            $this->secondToken = json_decode($response[1])->secondToken;
                            $this->event       = json_decode($response[1])->event;
                        } else {
                            $this->render = "success";
                            $this->setValidation();
                        }
                    }else{
                        $response = array();
                        $response[0] = '{"etat":0,"code":8000}';
                    }
                }
            }else{
                $uploads = JPATH_COMPONENT.'/public/';
                $dirname = dirname($uploads . json_decode($response[0])->fileName);
                $this->fileName = json_decode($response[0])->fileName;
                $this->fileContent = json_decode($response[0])->contentFile;
                $this->dirname = $dirname;
                $this->render = "writingerror";
            }
        }else{
            $response = array();
            $response[0] = '{"etat":0,"code":8000}';
        }
        $this->renderResponses($response);
    }
    
    /*
     * Handles connexion
     */
    public function connexion($objFormHandler) {
        $aFormHandler   = $objFormHandler->objToArray();
        //return   C:\wamp64\www\joomla\administrator/components/com_httpcsvalidation/public/
        $uploads      = JPATH_COMPONENT.'/public/';
        //  return /administrator/components/com_httpcsvalidation/public/
        $relativePath = str_replace(JPATH_SITE, '', $uploads);
        $this->email    = $objFormHandler->getEmail();
        $this->password = urldecode($objFormHandler->getPassword());

        $response[] = $this->curlRequest($this->PATHCHECKEMAILPASSWORD, array('url' => $aFormHandler['url'], 'relativePath' => $relativePath, 'password' => $aFormHandler['password'], 'email' => $this->email));
        if($response[0] != 'error'){
            if (!json_decode($response[0])->etat) {
                $this->fail = 1;
            } else {
                if (isset(json_decode($response[0])->fileName)) {
                    if($this->handleResponse($response[0])){
                        $response[] = $this->curlRequest($this->PATHCHECKFILE, array('url' => $aFormHandler['url'], 'relativePath' => $relativePath, 'token' => json_decode($response[0])->token, 'email' => $this->email, 'event' => 'connexion'));
                        if($response[1] != 'error'){
                            if (!json_decode($response[1])->etat) {
                                $this->render      = "retry";
                                $this->secondToken = json_decode($response[1])->secondToken;
                                $this->event       = json_decode($response[1])->event;
                            } else {
                                $this->render = "successco";
                                $this->setValidation();
                            }
                        }else{
                            $response = array();
                            $response[0] = '{"etat":0,"code":8000}';
                        }
                    }else{
                        $uploads = JPATH_COMPONENT.'/public/';
                        $dirname = dirname($uploads . json_decode($response[0])->fileName);
                        $this->fileName = json_decode($response[0])->fileName;
                        $this->fileContent = json_decode($response[0])->contentFile;
                        $this->dirname = $dirname;
                        $this->render = "writingerror";
                    }
                } else {
                    $this->render = "successco";
                    $this->setValidation();
                }
            }
        }else{
            $response = array();
            $response[0] = '{"etat":0,"code":8000}';
        }
        $this->renderResponses($response);
    }

    /*
     * Handles retry
     */
    public function retry($objFormHandler) {
        $aFormHandler      = $objFormHandler->objToArray();
        //return   C:\wamp64\www\joomla\administrator/components/com_httpcsvalidation/public/
        $uploads      = JPATH_COMPONENT.'/public/';
        //  return /administrator/components/com_httpcsvalidation/public/
        $relativePath = str_replace(JPATH_SITE, '', $uploads);
        $this->email       = $objFormHandler->getEmail();
        $this->secondToken = urldecode($objFormHandler->getSecondToken());
        $this->event       = urldecode($objFormHandler->getEvent());
        $response[]        = $this->curlRequest($this->PATHCHECKFILEAGAIN, array('url' => $aFormHandler['url'], 'relativePath' => $relativePath, 'secondToken' => $this->secondToken, 'email' => $this->email, 'event' => $this->event));
        if($response[0] != 'error'){
            if (!json_decode($response[0])->etat) {
                if(json_decode($response[0])->code == '9008'){
                    $this->render = "init";
                }else{
                    $this->render = "retry";
                }            
            } else {
                $this->render = "success";
                $this->setValidation();
            }
        }else{
            $response = array();
            $response[0] = '{"etat":0,"code":8000}';
        }
        $this->renderResponses($response);
    }
    
    /*
     * Handles responses of the web service and creates file
     */
    public function handleResponse($aResponse = array()) {
        if (!empty($aResponse)) {
            $aResult = json_decode($aResponse);
            $writingError = 1;
            if (isset($aResult->fileName) && isset($aResult->contentFile)) {
                $uploads = JPATH_COMPONENT.'/public/';
                $dirname = dirname($uploads . $aResult->fileName);
                if (!is_dir($dirname)) {
                    if(!mkdir($dirname, 0755, true)){
                        $writingError = 0;
                    }
                }
                if($writingError){
                    if(is_writable($uploads)){
                        $handleFile = fopen($uploads . $aResult->fileName, "w+");
                        if($handleFile){
                            fputs($handleFile, $aResult->contentFile);
                            fclose($handleFile);
                        }else{
                            $writingError = 0;
                        }
                    }else{
                        $writingError = 0;
                    }
                }
            }
            return $writingError;
        }
        return 0;
    }
    
    /*
     * Renders responses
     */
    public function renderResponses($aResponses) {
        if (!empty($aResponses)) {
            foreach ($aResponses as $aResponse) {
                if (!empty($aResponse)) {
                    $aResult = json_decode($aResponse);

                    if ($aResult->etat) {
                        $messageSuccess[] = $aResult->code;
                    } else {
                        $messageError[] = $aResult->code;
                    }
                }
            }

            if (isset($messageSuccess)) {
                $this->messageSuccess = $messageSuccess;
            }
            if (isset($messageError)) {
                $this->messageError = $messageError;
            }
        }
    }
    
    /*
     * Curl
     */
    public function curlRequest($urlHttpcs, $aFormHandler) {
        $curl     = curl_init();
        curl_setopt_array($curl, array(
            CURLOPT_URL            => $urlHttpcs,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_MAXREDIRS      => 5,
            CURLOPT_TIMEOUT        => 30,
            CURLOPT_HTTP_VERSION   => CURL_HTTP_VERSION_1_1,
            CURLOPT_POST           => true,
            CURLOPT_POSTFIELDS     => http_build_query($aFormHandler),
            CURLOPT_HEADER         => false,
            CURLOPT_USERAGENT      => 'plugin-joomla'
        ));
        $response = curl_exec($curl);
        $info = curl_getinfo($curl);
        if($response === false || empty($info) || $info['http_code'] != '200' ){
            curl_close($curl);
            return 'error';
        }
        //$err = curl_error($curl);
        curl_close($curl);
        return($response);
    }
    
    /*
     * Gets success and error messages
     */
    public function getNotices() {

        $notices = "";
        if (!empty($this->messageSuccess)) {
            $notices = '<div class="messages alert alert-success"><p>';
            foreach ($this->messageSuccess as $code) {
                $notices .= JText::_($this->renderNotices($code));
                $notices .= '<br>';
            }
            $notices .= '</p></div>';
        }
        if (!empty($this->messageError)) {
            $notices = '<div class="messages alert alert-danger"><p>';
            foreach ($this->messageError as $code) {
                $notices .= JText::_($this->renderNotices($code));
                if ($code == '9002') {
                    $notices .= ', <a id="coClick">' . JText::_('COM_HTTPCSVALIDATION_CLICK_HERE_LC') . '</a>';
                }
                if ($code == '9006') {
                    $notices .= ', <a id="creaClick">' . JText::_('COM_HTTPCSVALIDATION_CLICK_HERE_LC') . '</a>';
                }
                $notices .= '<br>';
            }
            $notices .= '</p></div>';            
        }
        return $notices;
    }
    
    /*
     * Gets translations
     */
    public function renderNotices($code) {
        include 'httpcsArrayLang.php';

        return $httpcsLang[$code];
    }
    
    /*
     * Store validation in database
     */
    public function setValidation(){
        $comparams = JComponentHelper::getParams('com_httpcsvalidation');
        $comparams->set('validated', '1');
        $componentid = JComponentHelper::getComponent('com_httpcsvalidation')->id;
        $table = JTable::getInstance('extension');
        $table->load($componentid);
        $table->bind(array('params' => $comparams->toString()));
        // check for error
        if (!$table->check()) {
            $this->setError('validated: check: ' . $table->getError());
            return false;
        }
        // Save to database
        if (!$table->store()) {
            $this->setError('validated: store: ' . $table->getError());
            return false;
        }
    }
    
    /*
     * Checks if Curl exists
     */
    public function isCurl() {
        return function_exists('curl_version');
    }
}